@@ -47,7 +47,7 @@ var proxiedFunctionInvokers = {};
 // the current compilation unit.
 // Also set for STANDALONE_WASM since the _start function is needed to call
 // static ctors, even if there is no user main.
-var HAS_MAIN = ('_main' in IMPLEMENTED_FUNCTIONS) || MAIN_MODULE || SIDE_MODULE || STANDALONE_WASM;
+var HAS_MAIN = ('_main' in IMPLEMENTED_FUNCTIONS) || MAIN_MODULE || STANDALONE_WASM;
 
 // Mangles the given C/JS side function name to assembly level function name (adds an underscore)
 function mangleCSymbolName(f) {
@@ -73,8 +73,7 @@ function JSify(data, functionsOnly) {
 
     var libFuncsToInclude;
     if (INCLUDE_FULL_LIBRARY) {
-      assert(!SIDE_MODULE, 'Cannot have both INCLUDE_FULL_LIBRARY and SIDE_MODULE set.')
-      libFuncsToInclude = (MAIN_MODULE || SIDE_MODULE) ? DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.slice(0) : [];
+      libFuncsToInclude = MAIN_MODULE ? DEFAULT_LIBRARY_FUNCS_TO_INCLUDE.slice(0) : [];
       for (var key in LibraryManager.library) {
         if (!isJsLibraryConfigIdentifier(key)) {
           libFuncsToInclude.push(key);
@@ -161,7 +160,7 @@ function JSify(data, functionsOnly) {
       if (allExternPrimitives.indexOf(ident) != -1) {
         usedExternPrimitives[ident] = 1;
         return;
-      } else if ((!LibraryManager.library.hasOwnProperty(ident) && !LibraryManager.library.hasOwnProperty(ident + '__inline')) || SIDE_MODULE) {
+      } else if (!LibraryManager.library.hasOwnProperty(ident) && !LibraryManager.library.hasOwnProperty(ident + '__inline')) {
         if (!(finalName in IMPLEMENTED_FUNCTIONS) && !LINKABLE) {
           var msg = 'undefined symbol: ' + ident;
           if (dependent) msg += ' (referenced by ' + dependent + ')';
@@ -193,7 +192,7 @@ function JSify(data, functionsOnly) {
             realIdent = realIdent.substr(2);
           }
 
-          var target = (SIDE_MODULE ? 'parent' : '') + "Module['" + mangleCSymbolName(realIdent) + "']";
+          var target = "Module['" + mangleCSymbolName(realIdent) + "']";
           var assertion = '';
           if (ASSERTIONS) {
             var what = 'function';
@@ -210,11 +209,6 @@ function JSify(data, functionsOnly) {
             functionBody = assertion + "return " + target + ".apply(null, arguments);";
           }
           LibraryManager.library[ident] = new Function(functionBody);
-          if (SIDE_MODULE) {
-            // no dependencies, just emit the thunk
-            Functions.libraryFunctions[finalName] = 1;
-            return processLibraryFunction(LibraryManager.library[ident], ident, finalName);
-          }
           noExport = true;
         }
       }
@@ -275,7 +269,7 @@ function JSify(data, functionsOnly) {
         if (typeof postset === 'function') {
           postset = postset();
         }
-        if (postset && !addedLibraryItems[postsetId] && !SIDE_MODULE) {
+        if (postset && !addedLibraryItems[postsetId]) {
           addedLibraryItems[postsetId] = true;
           itemsDict.GlobalVariablePostSet.push({
             JS: postset + ';'
@@ -400,21 +394,6 @@ function JSify(data, functionsOnly) {
     //
 
     if (!mainPass) {
-      if (!Variables.generatedGlobalBase) {
-        Variables.generatedGlobalBase = true;
-        // Globals are done, here is the rest of static memory
-        if (SIDE_MODULE) {
-          print('gb = alignMemory(getMemory({{{ STATIC_BUMP }}} + ' + MAX_GLOBAL_ALIGN + '), ' + MAX_GLOBAL_ALIGN + ' || 1);\n');
-          // The static area consists of explicitly initialized data, followed by zero-initialized data.
-          // The latter may need zeroing out if the MAIN_MODULE has already used this memory area before
-          // dlopen'ing the SIDE_MODULE.  Since we don't know the size of the explicitly initialized data
-          // here, we just zero the whole thing, which is suboptimal, but should at least resolve bugs
-          // from uninitialized memory.
-          print('for (var i = gb; i < gb + {{{ STATIC_BUMP }}}; ++i) HEAP8[i] = 0;\n');
-        }
-        // emit "metadata" in a comment. FIXME make this nicer
-        print('// STATICTOP = STATIC_BASE + ' + alignMemory(Variables.nextIndexedOffset) + ';\n');
-      }
       var generated = itemsDict.function.concat(itemsDict.type).concat(itemsDict.GlobalVariableStub).concat(itemsDict.GlobalVariable);
       print(generated.map(function(item) { return item.JS; }).join('\n'));
 
@@ -442,7 +421,7 @@ function JSify(data, functionsOnly) {
         if (USE_PTHREADS) {
           print('if (!ENVIRONMENT_IS_PTHREAD) {') // Pthreads should not initialize memory again, since it's shared with the main thread.
         }
-        print('/* memory initializer */ ' + makePointer(memoryInitialization, null, 'ALLOC_NONE', 'i8', 'GLOBAL_BASE' + (SIDE_MODULE ? '+H_BASE' : ''), true));
+        print('/* memory initializer */ ' + makePointer(memoryInitialization, null, 'ALLOC_NONE', 'i8', 'GLOBAL_BASE', true));
         if (USE_PTHREADS) {
           print('}')
         }
@@ -455,14 +434,12 @@ function JSify(data, functionsOnly) {
       return;
     }
 
-    var shellFile = SHELL_FILE ? SHELL_FILE : (SIDE_MODULE ? 'shell_sharedlib.js' : (MINIMAL_RUNTIME ? 'shell_minimal.js' : 'shell.js'));
+    var shellFile = SHELL_FILE ? SHELL_FILE : (MINIMAL_RUNTIME ? 'shell_minimal.js' : 'shell.js');
 
     var shellParts = read(shellFile).split('{{BODY}}');
     print(processMacros(preprocess(shellParts[0], shellFile)));
     var pre;
-    if (SIDE_MODULE) {
-      pre = processMacros(preprocess(read('preamble_sharedlib.js'), 'preamble_sharedlib.js'));
-    } else if (MINIMAL_RUNTIME) {
+    if (MINIMAL_RUNTIME) {
       pre = processMacros(preprocess(read('preamble_minimal.js'), 'preamble_minimal.js'));
     } else {
       pre = processMacros(preprocess(read('support.js'), 'support.js')) +
@@ -483,11 +460,9 @@ function JSify(data, functionsOnly) {
 
     legalizedI64s = legalizedI64sDefault;
 
-    if (!SIDE_MODULE) {
-      if (USE_PTHREADS) {
-        print('\n // proxiedFunctionTable specifies the list of functions that can be called either synchronously or asynchronously from other threads in postMessage()d or internally queued events. This way a pthread in a Worker can synchronously access e.g. the DOM on the main thread.')
-        print('\nvar proxiedFunctionTable = [' + proxiedFunctionTable.join() + '];\n');
-      }
+    if (USE_PTHREADS) {
+      print('\n // proxiedFunctionTable specifies the list of functions that can be called either synchronously or asynchronously from other threads in postMessage()d or internally queued events. This way a pthread in a Worker can synchronously access e.g. the DOM on the main thread.')
+      print('\nvar proxiedFunctionTable = [' + proxiedFunctionTable.join() + '];\n');
     }
 
     if (!MINIMAL_RUNTIME) {
@@ -539,7 +514,7 @@ function JSify(data, functionsOnly) {
       print(read('deterministic.js'));
     }
 
-    var postFile = SIDE_MODULE ? 'postamble_sharedlib.js' : (MINIMAL_RUNTIME ? 'postamble_minimal.js' : 'postamble.js');
+    var postFile = MINIMAL_RUNTIME ? 'postamble_minimal.js' : 'postamble.js';
     var postParts = processMacros(preprocess(read(postFile), postFile)).split('{{GLOBAL_VARS}}');
     print(postParts[0]);
 
@@ -547,13 +522,6 @@ function JSify(data, functionsOnly) {
 
     var shellParts = read(shellFile).split('{{BODY}}');
     print(processMacros(preprocess(shellParts[1], shellFile)));
-    // Print out some useful metadata
-    if (RUNNING_JS_OPTS) {
-      var generatedFunctions = JSON.stringify(keys(Functions.implementedFunctions));
-      if (RUNNING_JS_OPTS) {
-        print('// EMSCRIPTEN_GENERATED_FUNCTIONS: ' + generatedFunctions + '\n');
-      }
-    }
 
     PassManager.serialize();
   }
