@@ -54,29 +54,18 @@ Module['wasm'] = base64Decode('{{{ getQuoted("WASM_BINARY_DATA") }}}');
 
 #include "runtime_functions.js"
 #include "runtime_strings.js"
-#include "runtime_sab_polyfill.js"
 
 #if USE_PTHREADS
-var STATIC_BASE = {{{ GLOBAL_BASE }}};
-
 if (!ENVIRONMENT_IS_PTHREAD) {
 #endif
 
 var GLOBAL_BASE = {{{ GLOBAL_BASE }}},
     TOTAL_STACK = {{{ TOTAL_STACK }}},
-#if !USE_PTHREADS
-    STATIC_BASE = {{{ GLOBAL_BASE }}},
-#endif
     STACK_BASE = {{{ getQuoted('STACK_BASE') }}},
     STACKTOP = STACK_BASE,
     STACK_MAX = {{{ getQuoted('STACK_MAX') }}}
-#if USES_DYNAMIC_ALLOC
-    , DYNAMICTOP_PTR = {{{ DYNAMICTOP_PTR }}};
-#endif
     ;
 
-#if WASM
-
 #if ALLOW_MEMORY_GROWTH && MAXIMUM_MEMORY != -1
 var wasmMaximumMemory = {{{ MAXIMUM_MEMORY >>> 16 }}};
 #else
@@ -104,22 +93,8 @@ assert(buffer instanceof SharedArrayBuffer, 'requested a shared WebAssembly.Memo
 
 #include "runtime_init_table.js"
 
-#else
-
-#if USE_PTHREADS
-var buffer = new SharedArrayBuffer({{{ INITIAL_MEMORY }}});
-#else
-var buffer = new ArrayBuffer({{{ INITIAL_MEMORY }}});
-#endif
-
-#if USE_PTHREADS
-}
-#endif
-
-#endif
-
 #if ASSERTIONS
-var WASM_PAGE_SIZE = 65536;
+var WASM_PAGE_SIZE = {{{ WASM_PAGE_SIZE }}};
 #if USE_PTHREADS
 if (!ENVIRONMENT_IS_PTHREAD) {
 #endif
@@ -179,10 +154,6 @@ HEAPU8.set(new Uint8Array(Module['mem']), GLOBAL_BASE);
 
 #endif
 
-#if USES_DYNAMIC_ALLOC
-  HEAP32[DYNAMICTOP_PTR>>2] = {{{ getQuoted('DYNAMIC_BASE') }}};
-#endif
-
 #if USE_PTHREADS && ((MEM_INIT_METHOD == 1 && !MEM_INIT_IN_WASM && !SINGLE_FILE) || USES_DYNAMIC_ALLOC)
 }
 #endif
@@ -201,27 +172,6 @@ var wasmOffsetConverter;
 #endif
 
 #if EXIT_RUNTIME
-
-function callRuntimeCallbacks(callbacks) {
-  while(callbacks.length > 0) {
-    var callback = callbacks.shift();
-    if (typeof callback == 'function') {
-      callback();
-      continue;
-    }
-    var func = callback.func;
-    if (typeof func === 'number') {
-      if (callback.arg === undefined) {
-        dynCall_v(func);
-      } else {
-        dynCall_vi(func, callback.arg);
-      }
-    } else {
-      func(callback.arg === undefined ? null : callback.arg);
-    }
-  }
-}
-
 var __ATEXIT__    = []; // functions called during shutdown
 #endif
 
@@ -232,11 +182,6 @@ var runtimeInitialized = false;
 var runtimeExited = false;
 #endif
 
-/** @param {number|boolean=} ignore */
-{{{ unSign }}}
-/** @param {number|boolean=} ignore */
-{{{ reSign }}}
-
 #include "runtime_math.js"
 
 var memoryInitializer = null;
