@@ -28,7 +28,8 @@ logger = logging.getLogger('system_libs')
 LIBC_SOCKETS = ['socket.c', 'socketpair.c', 'shutdown.c', 'bind.c', 'connect.c',
                 'listen.c', 'accept.c', 'getsockname.c', 'getpeername.c', 'send.c',
                 'recv.c', 'sendto.c', 'recvfrom.c', 'sendmsg.c', 'recvmsg.c',
-                'getsockopt.c', 'setsockopt.c', 'freeaddrinfo.c']
+                'getsockopt.c', 'setsockopt.c', 'freeaddrinfo.c',
+                'in6addr_any.c', 'in6addr_loopback.c']
 
 
 def files_in_path(path_components, filenames):
@@ -74,9 +75,6 @@ def run_one_command(cmd):
   for opt in ['EMMAKEN_CFLAGS', 'EMMAKEN_JUST_CONFIGURE']:
     if opt in safe_env:
       del safe_env[opt]
-  # Disable certain warnings when we build ports/system libraries we don't want to
-  # show them a million times.
-  cmd.append('-Wno-fastcomp')
   # TODO(sbc): Remove this one we remove the test_em_config_env_var test
   cmd.append('-Wno-deprecated')
   shared.run_process(cmd, stdout=stdout, stderr=stderr, env=safe_env)
@@ -730,7 +728,7 @@ class libc(AsanInstrumentedLibrary, MuslInternalLibrary, MTLibrary):
     # Allowed files from ignored modules
     libc_files += files_in_path(
         path_components=['system', 'lib', 'libc', 'musl', 'src', 'time'],
-        filenames=['clock_settime.c'])
+        filenames=['clock_settime.c', 'asctime.c', 'ctime.c', 'gmtime.c', 'localtime.c'])
     libc_files += files_in_path(
         path_components=['system', 'lib', 'libc', 'musl', 'src', 'legacy'],
         filenames=['getpagesize.c', 'err.c'])
@@ -1333,14 +1331,14 @@ class libstandalonewasm(MuslInternalLibrary):
                    '__tm_to_secs.c',
                    '__tz.c',
                    '__year_to_secs.c',
+                   'asctime_r.c',
                    'clock.c',
                    'clock_gettime.c',
+                   'ctime_r.c',
                    'difftime.c',
                    'gettimeofday.c',
-                   'localtime.c',
-                   'localtime_r.c',
-                   'gmtime.c',
                    'gmtime_r.c',
+                   'localtime_r.c',
                    'nanosleep.c',
                    'mktime.c',
                    'time.c'])
@@ -1367,6 +1365,9 @@ class libjsmath(Library):
 # If main() is not in EXPORTED_FUNCTIONS, it may be dce'd out. This can be
 # confusing, so issue a warning.
 def warn_on_unexported_main(symbolses):
+  # In STANDALONE_WASM we don't expect main to be explictly exported
+  if shared.Settings.STANDALONE_WASM:
+    return
   if '_main' not in shared.Settings.EXPORTED_FUNCTIONS:
     for symbols in symbolses:
       if 'main' in symbols.defs:
@@ -1387,10 +1388,48 @@ def calculate(temp_files, in_temp, cxx, forced, stdout_=None, stderr_=None):
   if only_forced:
     temp_files = []
 
-  # Add in some hacks for js libraries. If a js lib depends on a symbol provided by a C library, it must be
-  # added to here, because our deps go only one way (each library here is checked, then we check the next
-  # in order - libc++, libcxextra, etc. - and then we run the JS compiler and provide extra symbols from
-  # library*.js files. But we cannot then go back to the C libraries if a new dep was added!
+  # deps_info.json is a mechanism that lets JS code depend on C functions. This
+  # needs special help because of how linking works:
+  #
+  #   1. Receive some input files (.o, .c, etc.) from the user.
+  #   2. Link them with system libraries.
+  #   3. Whatever symbols are still unresolved, look in JS libraries for them.
+  #
+  # This makes C->JS calls work in a natural way: if compiled code depends on
+  # a function foo() that is implemented in a JS library, it will be unresolved
+  # after stage 2, and therefore linked in at stage 3. The problem is the other
+  # direction: if a JS library function decides it needs some function from say
+  # libc, then at stage 3 it is too late to link in more libc code. That's
+  # where deps_info.json comes in.
+  #
+  # Specifically, before stage 2 (linking with system libraries) we look at what
+  # symbols are required by the input files. Imagine that js_func in a JS
+  # library depends on libc_func in libc. Then if deps_info.json tells us
+  #
+  #  "js_func": ["libc_func"]
+  #
+  # then if we see js_func is required (undefined) before stage 2, then we add
+  # a requirement to link in libc_func when linking in system libraries. All
+  # we do with deps_info.json is see if any of the keys are among the
+  # undefined symbols before stage 2, and if they are, add their values to the
+  # things we need to link in.
+  #
+  # This usually works the way you want, but note that it happens *before* stage
+  # 2 and not after it. That is, we look for js_func before linking in system
+  # libraries. If you have a situation where
+  #
+  #   user_code => other_libc_func => js_func => libc_func
+  #
+  # then the deps_info.json entry must contain
+  #
+  #  "other_libc_func": ["libc_func"]
+  #
+  # because that is what we see before stage 2: all we see is that
+  # other_libc_func is going to be linked in, and we don't know yet that it
+  # will end up calling js_func. But the presence of a call to other_libc_func
+  # indicates that we will need libc_func linked in as well, so that is what the
+  # deps_info.json entry should contain.
+  #
   # TODO: Move all __deps from src/library*.js to deps_info.json, and use that single source of info
   #       both here and in the JS compiler.
   deps_info = json.loads(open(shared.path_from_root('src', 'deps_info.json')).read())
